///////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// RECORDER.CPP - Definition of AKeystrokeRecorderApp, the Keystroke Recorder Application
//
// (c) 1998 by Stefan Ruck, Frankfurt/Main, Germany
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <os2.h>

#ifndef _IMSGBOX_
#include <imsgbox.hpp>
#endif

#ifndef _IRESLIB_
#include <ireslib.hpp>
#endif

#ifndef _IFONT_
#include <ifont.hpp>
#endif

#include "recorder.hpp"
#include "recorder.rch"
#include "about.hpp"

//*************************************************************************
// main  - Application entry point                                        *
//*************************************************************************
int main(int argc, char ** argv)
{
  AKeystrokeRecorderApp mainWindow (WND_MAIN); // Create our main window

  IApplication :: current (). run ();          // Get the current application and run it

  return (0);
}

//********************************************************************************************
// AKeystrokeRecorderApp :: AKeystrokeRecorderApp - Constructor of AKeystrokeRecorderApp     *
//********************************************************************************************
AKeystrokeRecorderApp :: AKeystrokeRecorderApp(unsigned long ulWindowID)
                         : IFrameWindow (IFrameWindow::titleBar
                                         | IFrameWindow::systemMenu
                                         | IFrameWindow::sizingBorder
                                         | IFrameWindow::minimizeButton
                                         | IFrameWindow::maximizeButton
                                         | IFrameWindow::windowList
                                         | IFrameWindow::appDBCSStatus
                                         | IFrameWindow::minimizedIcon
                                         | IFrameWindow::menuBar
                                         | IFrameWindow::accelerator,
                                         ulWindowID),
                           m_pEditField (0L),
                           m_pMenuBar (0L),
                           m_pKeyRecorderList (0L),
                           m_pStatusLine (0L),
                           m_bRecordKeystrokes (false),
                           m_bRecordingPaused (false)
{
  setupClientArea ();                            	// setup the variables

  ICommandHandler :: handleEventsFor (this);      // set self as command handler

  m_pEditField -> setFocus ();                    // set the focus on the edit field

  show ();                                        // show the window
}

//********************************************************************************************
// AKeystrokeRecorderApp :: ~AKeystrokeRecorderApp - Destructor of AKeystrokeRecorderApp     *
//********************************************************************************************
AKeystrokeRecorderApp :: ~AKeystrokeRecorderApp(void)

{
  // clean up the pointers
  delete m_pEditField;

  delete m_pMenuBar;

  delete m_pKeyRecorderList;

  delete m_pStatusLine;
}

//********************************************************************************
// AKeystrokeRecorderApp :: setupClientArea - set up the client area             *
//********************************************************************************
void AKeystrokeRecorderApp :: setupClientArea (void)
{
  // set a multiline-edit control as client

  m_pEditField = new IMultiLineEdit (WND_EDITFIELD, this, this);

  setClient (m_pEditField);

  // setup the menu bar

  m_pMenuBar = new IMenuBar (WND_MAIN, this);

  m_pMenuBar -> disableItem (MI_RECORDER_PAUSE);  // can't pause while not recording

  m_pMenuBar -> disableItem (MI_RECORDER_PLAY);   // actually nothing to playback

  // setup the status line, position it bwloq the client area, and set it's font to the menu's font

  m_pStatusLine = new IStaticText (WND_STATUSLINE, this, this);

  IPresSpaceHandle hps = m_pMenuBar -> presSpace();

  IFont * pMenueFont = new IFont (hps);

  releasePresSpace (hps);

  addExtension (m_pStatusLine, IFrameWindow :: belowClient, 
                pMenueFont -> maxAscender() + (ULONG) 2 * pMenueFont -> maxDescender ());

  m_pStatusLine -> setFont (*pMenueFont);

  delete pMenueFont;
}



//********************************************************************************
// AKeystrokeRecorderApp :: command - handles Command-Events                     *
//********************************************************************************
Boolean AKeystrokeRecorderApp :: command(ICommandEvent &evt)
{
  Boolean bFiltered = true;                                        // event handled

  switch(evt.commandId())                      	                  // handle the menu commands
         {
          case MI_FILE_EXIT:                                      // exit the program
               close ();                                          // close the framewindow
               break;
          case MI_RECORDER_RECORD:                                // start/stop the keystroke recorder
               recorderStartStop ();
               break;
          case MI_RECORDER_PAUSE:                                // pause/resume the keystroke recorder
               recorderPauseResume ();
               break;
          case MI_RECORDER_PLAY:                                  // play the recorded keystrokes
               recorderPlay ();
               break;
          case MI_HELP_ABOUT:                                     // show the product information
               {
                AAbout * pDlgAbout = new AAbout (this);           
                pDlgAbout -> showModally ();                      
                delete pDlgAbout;                                 
                break;
               }
          default:
              bFiltered = false;                                   // event not handled
              break;
         }

  return (bFiltered);
}

//********************************************************************************
// AKeystrokeRecorderApp :: recorderStartStop - starts or stops the key recorder *
//********************************************************************************
void AKeystrokeRecorderApp :: recorderStartStop ()

{
  if (m_bRecordKeystrokes)                              // do we already record the keystrokes?
      {                                                 // yes, so stop it, reset the flags
       m_bRecordKeystrokes = false;
       m_bRecordingPaused = false;
       // reset the menu
       m_pMenuBar -> setText (MI_RECORDER_RECORD, IResourceId (STR_START_RECORDER));  
       m_pMenuBar -> enableItem (MI_RECORDER_PLAY, !m_pKeyRecorderList -> isEmpty ()); // only enabled if list is not empty
       m_pMenuBar -> enableItem (MI_RECORDER_PAUSE, false);
       m_pMenuBar -> setText (MI_RECORDER_PAUSE, IResourceId (STR_PAUSE_RECORDER));
       IKeyboardHandler :: stopHandlingEventsFor (m_pEditField);  // stop the keyboard-event handling
       m_pStatusLine -> setText ("");                            // clear the text of the status line
       return;
      }

  // we are not recording, so do it

  delete m_pKeyRecorderList;                            // clean up the old keystrokes

  m_pKeyRecorderList = 0L;

  m_pKeyRecorderList = new ISequence<IKeyboardEvent>;   // get a new list

  // update the menu
  m_pMenuBar -> setText (MI_RECORDER_RECORD, IResourceId (STR_STOP_RECORDER));

  m_pMenuBar -> enableItem (MI_RECORDER_PLAY, false);

  m_pMenuBar -> enableItem (MI_RECORDER_PAUSE, true);

  m_bRecordKeystrokes = true;                           // set the flags to the current values
  m_bRecordingPaused = false;

  IKeyboardHandler :: handleEventsFor (m_pEditField);   // handling the keyboard-events of the edit field so we can record them

  m_pStatusLine -> setText (IResourceId (STR_RECORDING));                    // set the text of the status line
}


//********************************************************************************
// AKeystrokeRecorderApp :: recorderPauseResume - pause/resume the recorder      *
//********************************************************************************
void AKeystrokeRecorderApp :: recorderPauseResume ()

{
  if (m_bRecordingPaused)  // is the pause active?
      {
       // yes, resume the recording
       m_pMenuBar -> setText (MI_RECORDER_PAUSE, IResourceId (STR_PAUSE_RECORDER)); // switch the menu text
       IKeyboardHandler :: handleEventsFor (m_pEditField);            // resume the keyboard-event handling
       m_bRecordingPaused = false;                                                          // set the flag
       m_pStatusLine -> setText (IResourceId (STR_RECORDING));           // set the text of the status line
      }
  else
      {
       // no, so pause the recording
       m_pMenuBar -> setText (MI_RECORDER_PAUSE, IResourceId (STR_RESUME_RECORDER)); // switch the menu text
       IKeyboardHandler :: stopHandlingEventsFor (m_pEditField);         // stop the keyboard-event handling
       m_bRecordingPaused = true;                                                            // set the flag
       m_pStatusLine -> setText (IResourceId (STR_PAUSED));               // set the text of the status line
      }
}

//********************************************************************************
// AKeystrokeRecorderApp :: recorderPlay - replay the recorded key strokes       *
//********************************************************************************
void AKeystrokeRecorderApp :: recorderPlay ()

{
  if (!m_pKeyRecorderList                   // do we have recorded key strokes?
      || m_pKeyRecorderList -> isEmpty ())
      return;                               // no, so return

  m_pStatusLine -> setText (IResourceId (STR_PLAY));             // set the text of the status line

  ICursor * pCursor = m_pKeyRecorderList -> newCursor ();              // get a cursor for the list

  m_pKeyRecorderList -> setToFirst (*pCursor);

  do                                          // send all the recorded keystrokes to the edit field
    {
     m_pEditField -> sendEvent (m_pKeyRecorderList -> elementAt (*pCursor));
    }
     while (m_pKeyRecorderList -> setToNext (*pCursor));       // no more events, so break the loop

  delete pCursor;                                                                       // clean up

  m_pStatusLine -> setText ("");                               // clear the text of the status line
}

//*****************************************************************************************
// AKeystrokeRecorderApp :: handleKeyPress - standard key press handling                  *
//*****************************************************************************************
Boolean AKeystrokeRecorderApp :: handleKeyPress (IKeyboardEvent & event)

{
  m_pKeyRecorderList -> addAsLast (event);                  // add the event to the list

  IKeyboardHandler :: stopHandlingEventsFor (m_pEditField); // pause from event handling to avoid loop

  m_pEditField -> sendEvent (event);                        // send the event to the edit field

  IKeyboardHandler :: handleEventsFor (m_pEditField);       // resume the event handling

  return (true);
}


